package com.zerog.ia.customcode.actions;

import com.zerog.ia.api.pub.*;
import java.io.*;

/**
 * ExecuteUninstallCommand.java
 *
 * This custom code action execute a command during uninstall.
 *
 */
public class ExecuteUninstallCommand extends CustomCodeAction
{
    /**
     *  This is the method that is called at install-time.
     */
    public void install( InstallerProxy ip ) throws InstallException
    {

        // Write command to execute to DataOuput for use during uninstall.
        try
        {
            // Get the value of the "$COMMAND_TO_EXECUTE$" InstallAnywhere variable
            String executeString = ip.substitute("$COMMAND_TO_EXECUTE$");

            if(executeString.equals(""))
                System.err.println("ExecuteUninstallCommand: $COMMAND_TO_EXECUTE$ not set!");

            DataOutput da = ip.getLogOutput();

            da.writeUTF(executeString);

        }
        catch(Exception e)
        {
            System.err.println("ExecuteUninstallCommand: ");
            e.printStackTrace();
            throw new NonfatalInstallException("ExecuteUninstallCommand ERROR");

        }
    }


    /**
     *  This is the method that is called at uninstall-time.
     */
    public void uninstall( UninstallerProxy up ) throws InstallException
    {
        try
        {
            DataInput di = up.getLogInput();

            // Read in command to execute from DataInput
            String executeString = di.readUTF();

            //System.out.println("ExecuteUninstallCommand: About to execute: " + executeString);
            final Process p = Runtime.getRuntime().exec(executeString);

            // Intialize StringWriters that capture the output from the command
            final StringWriter errWriter = new StringWriter();
            final StringWriter outWriter = new StringWriter();

            // Launch a thread to capture the Standard Error Output of the Command
            Thread thread = new Thread()
            {
                public void run()
                {
                    InputStream err = p.getErrorStream();
                    int c;
                    try
                    {
                        while ((c = err.read()) >= 0)
                        {
                            errWriter.write(c);
                        }
                    }
                    catch (IOException ioe)
                    {
                    }
                }
            };

            // Launch a thread to capture the Standard Output of the Command
            Thread thread2 = new Thread()
            {
                public void run()
                {
                    InputStream err = p.getInputStream();
                    int c;
                    try
                    {
                        while ((c = err.read()) >= 0)
                        {
                            outWriter.write(c);
                        }
                    }
                    catch (IOException ioe)
                    {
                    }
                }
            };

            thread.start();
            thread2.start();
            p.waitFor();

            // Take the information captured from the string writers, and write them out.
            String stdErr   = errWriter.toString();
            String stdOut   = outWriter.toString();

            // Get the return code
            String exitCode = new Integer(p.exitValue()).toString();

            // Print out captured info (for debugging)

            System.out.println("\n****************************************");
            System.out.println("ExecuteUninstallCommand:" + executeString);
            System.out.println("\nStdErr Output: " + stdErr);
            System.out.println("\nStdOut Output: " + stdOut);
            System.out.println("\nExit Code: " + exitCode);
            System.out.println("****************************************\n");

        }
        catch(Exception e)
        {
            System.err.println("ExecuteUninstallCommand: ");
            e.printStackTrace();
        }
    }


    /**
     *  This method will be called to display a status message during the
     *  installation.
     */
    public String getInstallStatusMessage()
    {
        return "ExecuteUninstallCommand";
    }

    /**
     *  This method will be called to display a status message during the
     *  uninstall.
     */
    public String getUninstallStatusMessage()
    {
        return "ExecuteUninstallCommand";
    }
}